// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

// Sat 06-15-1996

#ifndef __COLORFUNCTION_H
#define __COLORFUNCTION_H 1

#include <iostream>
#include <function.h>

//-----------------------------------------------------------------------//
//                          OneColorFunction
//-----------------------------------------------------------------------//
//
// This class represents a function f:R->R^3 that returns a constant RGB-value

class OneColorFunction: public Function3D {
  protected:
    double3 RGB;
    
  public:
    OneColorFunction(): RGB(1,0,0) {}
    // default constructor

    OneColorFunction(const double3& rgb): RGB(rgb) {}
    // default constructor
    
    OneColorFunction(const OneColorFunction& f) { RGB = f.RGB; }
    // copy constructor

    void SetRed()    { RGB = double3(1,0,0); }
    void SetGreen()  { RGB = double3(0,1,0); }
    void SetBlue()   { RGB = double3(0,0,1); }
    void SetYellow() { RGB = double3(1,1,0); }

    double3 operator()(double t) const { return RGB; }
    // returns f(t)
};

//-----------------------------------------------------------------------//
//                          TwoColorFunction
//-----------------------------------------------------------------------//
//
// This class represents a linear function f:R->R^3 that interpolates
// between two RGB-values

class TwoColorFunction: public Function3D {
  protected:
    double3 RGB0, RGB1;
    
  public:
    TwoColorFunction(): RGB0(1,0,0), RGB1(0,1,0) {}
    // default constructor

    TwoColorFunction(const double3& rgb0, const double3& rgb1): RGB0(rgb1), RGB1(rgb1) {}
    // constructor
    
    void SetYellowRed() { RGB0 = double3(1,1,0); RGB1 = double3(1,0,0); }
    void SetWhiteBlue() { RGB0 = double3(0,0,1); RGB1 = double3(1,1,1); }

    double3 operator()(double t) const { return (1-t)*RGB0 + t*RGB1; }
    // returns f(t)
};

#endif

