// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

// Sun 06-30-1996

#include <double3.h>
#include <polyline.h>

void VRMLWriteColor(std::ostream& out, const double3& rgb);
// This function writes an RGB color.

void VRMLWriteTranslation(std::ostream& out, const double3& x);
// This function writes a translation over the vector x.

void VRMLWriteAxialRotation(std::ostream& out, const double3& n, double phi);
// This function writes a rotation around the axis n over the angle phi.

void VRMLWriteVectorRotation(std::ostream& out, const double3& a, const double3& b);
// This function writes a rotation that maps a/a.Unit() to b/b.Unit().

void VRMLWriteCylinder(std::ostream& out, const double3& x0, const double3& x1, double r);
void VRMLWriteCylinder(std::ostream& out, const double3& x0, const double3& x1, double r, const double3& color);
// This function writes a cylinder with central axis (x0,x1) and radius r.

void VRMLWriteSphere(std::ostream& out, const double3& m, double r);
void VRMLWriteSphere(std::ostream& out, const double3& m, double r, const double3& color);
// This function writes a sphere with center m and radius r.

void VRMLWriteCone(std::ostream& out, const double3& bottom, const double3& top, double r);
void VRMLWriteCone(std::ostream& out, const double3& bottom, const double3& top, double r, const double3& color);
// This function writes a cone with central axis (bottom,top) and radius r.

void VRMLWritePolygon(std::ostream& out, const double3& p0, const double3& p1, const double3& p2, const double3& p3);
void VRMLWritePolygon(std::ostream& out, const double3& p0, const double3& p1, const double3& p2, const double3& p3, const double3& color);
// This function writes a polygon containing 4 points.

void VRMLWriteLine(std::ostream& out, const double3& x0, const double3& x1);
void VRMLWriteLine(std::ostream& out, const double3& x0, const double3& x1, const double3& color);
// This function writes a line between x0 an x1.

void VRMLWriteArrow(std::ostream& out, const double3& x0, const double3& x1, double r);
void VRMLWriteArrow(std::ostream& out, const double3& x0, const double3& x1, double r, const double3& color);
// This function writes an arrow with consisting of a cylinder and a cone. The radius
// of the cylinder is r.

void VRMLWritePolylineAsCylinders(std::ostream& out, const Polyline3D& poly, double r, const double3& color);
// This function draws a polyline as a sequence of cylinders with radius r.

void VRMLWriteFileHeader(std::ostream& out);
// This function writes the string "#VRML V1.0 ascii".

void VRMLWriteBeginSeparator(std::ostream& out, int numspaces=0);
void VRMLWriteEndSeparator(std::ostream& out, int numspaces=0);

