// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

#include <cstdlib>
#include <iostream>
#include <fstream>
#include <sstream>
#include <cmath>
#include <bvalues.h>
#include <dmatrix.h>
#include <dvector.h>
#include "tbspline.h"
#include "quadfunc.h"

//-----------------------------------------------------------------------//
//                          GetFileName                                  //
//-----------------------------------------------------------------------//
std::string GetFileName(std::string InFile, int i)
// creates the name InFile_i
{
  std::stringstream out;
  out << InFile << "_" << std::setw(3) << std::setfill('0') << i << std::ends;
  return out.str();
}

//-----------------------------------------------------------------------//
//                          main                                         //
//-----------------------------------------------------------------------//
int main(int argc, char* argv[])
// usage: FileName, maxiter
{
  int maxiter;
  char FileName[80];

  std::cout << "Iterating surface" << std::endl;

  if (argc > 1) {
    strcpy(FileName, argv[1]);
  }
  else {
    std::cout << "input file: "; std::cin >> FileName;
  }

  if (argc > 2) {
    maxiter = atoi(argv[2]);
  }
  else {
    std::cout << "number of iterations: "; std::cin >> maxiter;
  }

  if (maxiter <= 0) {
    std::cerr << "Number of iterations must be >= 0!" << std::endl;
    std::exit(0);
  }

  TensorBSpline F(FileName);

  EquationSystem Constraints;
  F.GetFixedBoundaryEquations(Constraints);

  DoubleMatrix C;
  DoubleVector D;

  Constraints.GetMatrix(C);
  Constraints.GetVector(D);
  std::cout << "C=" << C << std::endl;
  std::cout << "D=" << D << std::endl;

  double px, py, pz;
  std::cout << "interpolation point:"; std::cin >> px >> py >> pz;
  double3 P(px,py,pz);
  double u = (F.MinUPar() + F.MaxUPar())/2;
  double v = (F.MinVPar() + F.MaxVPar())/2;
  F.GetInterpolatePointEquation(u, v, P, Constraints);

  Constraints.GetSimpleForm(C,D);

  DoubleMatrix fA;
  DoubleVector fB(F.NumControlPoints(), 0);
  double fC = 0;

  for (int i=0; i<maxiter; i++) {
    // compute the total curvature fairness matrix with F as a reference surface
    F.GetTotalCurvatureMatrix(F, fA);
    QuadraticFunction f(new SparseDoubleMatrix(fA), new SparseDoubleVector(fB), fC);

    DoubleVector X;
    X = F.GetControlVector();

    std::cout << i << " total curvature = " << f(X) << std::endl;

    f.GetConstrainedMinimum(C, D, X);
    F.SetControlVector(X);

    std::string OutFileName = GetFileName(FileName, i+1);
    F.SaveFile(OutFileName);
  }
}

