// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

#include <cstdlib>
#include "vbspline.h"

#include "constrai.h"
#include "potentia.h"

const double FACTOR = 5.0;

int main(int argc, char *argv[])
{
  int N = 7;           // aantal controle punten
  double alpha = 0.5;  

  VariationalBSpline curve;

  // stop random controle punten in de curve
  for (int i=0; i<N; i++) {
    double3* point = new double3(
      double3((100.0*i)/N, 0, 0) + FACTOR*double3(drand48(), drand48(), drand48())
    );
    curve.InsertControlPoint(point);
  }

  // laat de controle punten zien
  for (i=0; i<curve.NumControlPoints(); i++)
    std::cout << i << " " << curve.ControlPoint(i) << std::endl;

  // creeer een punt attractor
  double t0 = 2 + 0.4*(N-3);
  double3 point(1,2,3);
  double weight = 1;
  SparseQuadraticFunction* pointattr = curve.GetPointToPointAttractor(weight, t0, point);

  DoubleMatrix A;
  DoubleVector B;
  double C;
  pointattr->GetMatrices(A,B,C); 
  // de bijdrage van de puntattractor is X^T A X + B^T X + C

  // controle of het klopt
  DoubleVector X = curve.GetControlVector();
  std::cout << (*pointattr)(X) << std::endl;
  std::cout << (X, A*X) + (B,X) + C << std::endl;
  double3 cp = curve.GetPoint(t0);
  std::cout << weight*(cp-point,cp-point) << std::endl;
  
  BandMatrix* Astretch = curve.GetSimpleStretchEnergy(alpha);
  BandMatrix* Abend = curve.GetSimpleBendEnergy(1-alpha);

  // voeg interne energie termen toe aan de door A,B en C gedefinieerde
  // kwadratische functie 
  Astretch->AddToDoubleMatrix(A);
  Abend->AddToDoubleMatrix(A);

  // constraints
  double beginpar = curve.MinPar();
  double3 beginpoint(0,0,0);
  double endpar = curve.MaxPar(); 
  double3 endpoint(10,0,0);

  SparseConstraint* constr1 = curve.GetPointToPointConstraint(beginpar, beginpoint);
  SparseConstraint* constr2 = curve.GetPointToPointConstraint(endpar, endpoint);

  DoubleEquationSystem eqn;
  constr1->AddToEquationSystem(eqn);
  constr2->AddToEquationSystem(eqn);
  
  // laat de constraint matrices zien: D X = E
  DoubleMatrix D;
  DoubleVector E;
  eqn.GetMatrix(D);
  eqn.GetVector(E);
  std::cout << "D = " << D << std::endl;
  std::cout << "E = " << E << std::endl;

  // herschrijf de constraints tot: X = FY + G 
  // zodanig dat DF = 0 en DG = E
  DoubleMatrix F;
  DoubleVector G;
  eqn.GetSimpleForm(F, G);
  std::cout << "F = " << F << std::endl;
  std::cout << "G = " << G << std::endl;
  std::cout << "DF = " << D*F << std::endl;
  std::cout << "DG - E = " << D*G - E << std::endl;

/*
  // code om het lineaire stelsel AX = B direct op te lossen:
  DoubleSymFact LU(A);
  DoubleVector X = LU.solve(A);
  
  // code om de eigenwaarden van matrix A te bepalen:
  DoubleVector ev = A.EigenValues();
  ev.Sort(); // sorteren op grootte
  
  // code om de kwadratische functie X^T A X + B^T X + C te minimaliseren:
  QuadraticFunction f(A.NumRows());
  f.AddTerm(A,B,C);
  f.GetMinimum(X); 

  // hetzelfde, onder voorwaarde dat X = FY + G
  f.GetConstrainedMinimum(F, G, X);
*/
  
  return 0;
}

