// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

#include <cstdlib>
#include "vbspline.h"
#include <bvalues.h>
#include <integr.h>
#include <geotyp.h>

#include "constrai.h"
#include "potentia.h"

void Test(VariationalBSpline& curve)
{
  DoubleVector X = curve.GetControlVector();

  curve.Show();

  // method 1
  QuadraticFunctionList b1 = curve.GetSimpleBendEnergy(1, 0, 1);
  QuadraticFunctionList s1 = curve.GetSimpleStretchEnergy(1, 0, 1);
  double Eb1 = b1(X);
  double Es1 = s1(X);

  // method 2
  BandMatrix* Astretch = curve.GetSimpleStretchEnergy(1);
  BandMatrix* Abend = curve.GetSimpleBendEnergy(1);
  double Eb2 = (X, (*Abend)*X);
  double Es2 = (X, (*Astretch)*X);

  double Eb3 = curve.ComputeSimpleBendEnergy(7);
  double Es3 = curve.ComputeSimpleStretchEnergy(7);

  std::cout << "Es1 = " << std::setw(15) << Es1 << std::endl;
  std::cout << "Es2 = " << std::setw(15) << Es2 << std::endl;
  std::cout << "Es3 = " << std::setw(15) << Es3 << std::endl;
  std::cout << "Eb1 = " << std::setw(15) << Eb1 << std::endl;
  std::cout << "Eb2 = " << std::setw(15) << Eb2 << std::endl;
  std::cout << "Eb3 = " << std::setw(15) << Eb3 << std::endl;
}

int main(int argc, char *argv[])
{
  std::ifstream from(argv[1]);
  if (!from) {
    std::cout << "Could not open file " << argv[1] << std::endl;
    exit(1);
  }

  // define a curve
  VariationalBSpline curve;
  from >> curve;
  from.close();

  Test(curve);

  return 0;
}

