// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

#include <cstdlib>
#include "vbspline.h"
#include <bvalues.h>
#include <integr.h>
#include <sample.h>
#include <geotyp.h>

int main(int argc, char *argv[])
{
  std::ifstream from(argv[1]);
  if (!from) {
    std::cout << "Could not open file " << argv[1] << std::endl;
    exit(1);
  }

  // define a curve
  VariationalBSpline curve;
  from >> curve;
  from.close();

  double t = 0.33;
  double3 L = curve.LaplaceBeltrami(t,curve);
  double kappa1 = (L,L);
  double kappa2 = curve.Curvature(t); kappa2 = kappa2*kappa2;
  std::cout << "kappa1 = " << kappa1 << std::endl;
  std::cout << "kappa2 = " << kappa2 << std::endl;

  double l1 = curve.ComputeStretchEnergy();
  double l2 = curve.ComputeStretchEnergyRef();
  double k1 = curve.ComputeBendEnergy();
  double k2 = curve.ComputeBendEnergyRef();

  std::cout << "l1 = " << l1 << std::endl;
  std::cout << "l2 = " << l2 << std::endl;
  std::cout << "k1 = " << k1 << std::endl;
  std::cout << "k2 = " << k2 << std::endl;

  return 0;
}

