// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

#include <iostream>
#include <fstream>
#include <sstream>
#include <cmath>
#include <string>

#include <bvalues.h>
#include <dmatrix.h>
#include <dvector.h>
#include <function3.h>
#include <colorfnc.h>
#include <d3equation.h>

#include "tbspline.h"

//-----------------------------------------------------------------------//
//                          WriteVRML                                    //
//-----------------------------------------------------------------------//
void WriteVRML(TensorBSpline& F, char* FileName)
{
  SurfTriangulation tri;
  F.GetTriangulation(50,50,tri);
  tri.SetTotalCurvatureValues();
  tri.ScaleValues(0,1);

  YellowToRed colorfunc;

  tri.SaveVRML(FileName, colorfunc);
}

//-----------------------------------------------------------------------//
//                          GetFileName                                  //
//-----------------------------------------------------------------------//
std::string GetFileName(const char* InFile, int i)
// creates the name InFile_i
{
  std::stringstream out;
  out << InFile << "_" << std::setw(3) << std::setfill('0') << i << std::ends;
  return out.str();
}

//-----------------------------------------------------------------------//
//                          main                                         //
//-----------------------------------------------------------------------//
int main(int argc, char* argv[])
{
  int maxiter;

  std::cout << "maxiter: "; std::cin >> maxiter;

  char FileName[] = "itersurf";

/*
  char FileName[80];

  std::cout << "Iterating surface" << std::endl;

  if (argc > 1) {
    strcpy(FileName, argv[1]);
  }
  else {
    std::cout << "input file: "; std::cin >> FileName;
  }

  if (argc > 2) {
    maxiter = atoi(argv[2]);
  }
  else {
    std::cout << "number of iterations: "; std::cin >> maxiter;
  }

  if (maxiter <= 0) {
    std::cerr << "Number of iterations must be >= 0!" << std::endl;
    exit(0);
  }

  TensorBSpline F(FileName);
*/

  const int NumRows = 8;    // number of control points in u direction
  const int NumCols = 5;    // number of control points in v direction
  const double Height = 10;
  const double Radius = 5;
  const int Angle = 180;
 
  // define a test surface
  TensorBSpline F;
  F.SetCylinder(NumRows, NumCols, Height, Radius, Angle);

  // apply constraints to surface F
  Double3EquationSystem Constraints;
  F.GetFixedBoundaryEquations(Constraints);
  DoubleMatrix D;
  DoubleVector E;
  double3 P(0,0,5);
  double u = (F.MinUPar() + F.MaxUPar())/2;
  double v = (F.MinVPar() + F.MaxVPar())/2;
  F.GetInterpolatePointEquation(u, v, P, Constraints);
  Constraints.GetSimpleForm(D,E);   // constraints: X = DY + E

  int n = F.NumControlPoints();
  DoubleMatrix A(n,n); A.MakeZero();
  DoubleVector B(3*n); B.MakeZero();
  double C = 0;

  for (int i=0; i<maxiter; i++) {
    // compute the total curvature fairness matrix with F as a reference surface
    F.GetTotalCurvatureMatrix(F, A);

    DoubleVector X;
    X = F.GetControlVector();

    QuadraticFunctionXYZ f(A,B,C);  // N.B. makes copies of A,B en C!!
    std::cout << i << " total curvature = " << f(X) << std::endl;

    f.GetConstrainedMinimum(D, E, X);
    F.SetControlVector(X);

    std::string OutFileName = GetFileName(FileName, i+1);
    WriteVRML(F, OutFileName.c_str());
  }

  return 0;
}

