// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

// 23-3-95

#include <iostream>
#include "fixpoint.h"

//-----------------------------------------------------------------------//
//                          TFixedPointHandler                           //
//-----------------------------------------------------------------------//
TFixedPointHandler::TFixedPointHandler():
  fixedpoints(10,10),
  numpoints(0)
{}

TFixedPointHandler::TFixedPointHandler(int np):
  fixedpoints(10,10)
{
  SetSize(np);
}

DoubleVector TFixedPointHandler::GetFreePointsVector(DoubleVector& X)
{
  DoubleVector Y(3*NumFreePoints());

  for (int i=0; i<numpoints; i++) {
    if (!fixed[i]) {
      int xrow = 3*i;
      int yrow = 3*index[i];
      Y[yrow]   = X[xrow];
      Y[yrow+1] = X[xrow+1];
      Y[yrow+2] = X[xrow+2];
    }
  }

  return Y;
}

DoubleVector TFixedPointHandler::GetAllPointsVector(DoubleVector& Y)
{
  DoubleVector X(3*NumPoints());

  for (int i=0; i<numpoints; i++) {
    if (fixed[i]) {
      double3& Point = fixedpoints[index[i]];
      int row = 3*i;
      X[row]   = Point.x();
      X[row+1] = Point.y();
      X[row+2] = Point.z();
    }
    else {
      int xrow = 3*i;
      int yrow = 3*index[i];
      X[xrow]   = Y[yrow];
      X[xrow+1] = Y[yrow+1];
      X[xrow+2] = Y[yrow+2];
    }
  }

  return X;
}

void TFixedPointHandler::FixPoint(int i, const double3& point)
{
  if (fixed[i] == 1)
    fixedpoints[GetIndex(i)] = point;
  else {
    fixed[i] = 1;

    // compute new index for point i
    index[i] = 0;
    for (int j=0; j<i; j++)
      index[i] += fixed[j];

    // compute new index for points i+1 .. numpoints -1
    for (int j=i+1; j<numpoints; j++)
      if (fixed[j] == 1)
        index[j]++;
      else
        index[j]--;

    fixedpoints.AtInsert(index[i], new double3(point));
  }
}

void TFixedPointHandler::FreePoint(int i)
{
  if (fixed[i] == 1) {
    fixed[i] = 0;
    fixedpoints.AtDelete(index[i]);

    // compute new index for point i
    index[i] = i;
    for (int j=0; j<i; j++)
      index[i] -= fixed[j];

    // compute new index for points i+1 .. numpoints -1
    for (int j=i+1; j<numpoints; j++)
      if (fixed[j] == 1)
        index[j]--;
      else
        index[j]++;
  }
}

void TFixedPointHandler::SetSize(int np)
{
  numpoints = np;

  fixed.SetSize(numpoints);
  fixed.MakeZero();

  index.SetSize(numpoints);
  index.MakeSequence();
}

void TFixedPointHandler::Show()
{
  std::cout << "TFixedPointHandler::Show()" << std::endl;
  std::cout << "fixed = " << fixed << std::endl;
  std::cout << "index = " << index << std::endl;
  std::cout << "numpoints = " << numpoints << std::endl;
  std::cout << "numfixedpoints = " << NumFixedPoints() << std::endl;
  std::cout << "fixedpoints: " << std::endl;
  for (int i=0; i<fixedpoints.Count; i++)
    std::cout << i << " " << fixedpoints[i] << std::endl;
}

