// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

// Sat 06-22-1996

#include "consurf.h"
#include <mystuff.h>

BoundingBox3D ControlPointSurface::GetControlPointBBox() const
// returns the bounding box of the control points
{
  BoundingBox3D box;

  for (int i=0; i<NumControlPoints(); i++) 
    box.Insert(GetControlPoint(i)); 

  return box;
}

void ControlPointSurface::GetControlPointMesh(PolygonMesh& mesh) const
// generates a mesh of the control polyhedron
{
  std::cerr << "ControlPointSurface::GetControlPointMesh() not yet implemented!";
}

DoubleVector ControlPointSurface::GetControlPointVector() const
// computes the concatenation vector of the control points
{ 
  DoubleVector x(3*NumControlPoints());

  for (int i=0; i<NumControlPoints(); i++) {
    const double3& point = GetControlPoint(i);
    int row = 3*i;
    x[row]   = point.x();
    x[row+1] = point.y();
    x[row+2] = point.z();
  }
  
  return x;
}

void ControlPointSurface::SetControlPointVector(DoubleVector& x)
// initializes the control points with the concatenation vector x
{
  for (int i=0; i<NumControlPoints(); i++) {
    int row = 3*i;
    SetControlPoint(i, double3(x[row], x[row+1], x[row+2]));
  }
}

void ControlPointSurface::Perturb(double d)
{
  for (int i=0; i<NumControlPoints(); i++) {
    double3 point(Random(-d,d), Random(-d,d), Random(-d,d));
    SetControlPoint(i, point);
  }
}

