// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

// Thu 06-13-1996

#include <integr.h>

TIntPointEvaluator IntPointEvaluator;

TIntPointEvaluator::TIntPointEvaluator():
  Interval1_1(1,1,1), Interval3_1(2,1,1), Interval5_1(3,1,1), Interval7_1(4,1,1), Interval9_1(5,1,1),
  Simplex2_2(3,1,1), Simplex2_3(6,1,1), Simplex2_5(7,1,1), Simplex2_7(16,1,1),
  Cube3_1(4,1,1), Cube5_1(7,1,1), Cube7_1(12,1,1), Cube9_1(20,1,1)
{
  std::cout << "Initializing integration points..." << std::flush;
  InitCubePoints();
  InitSimplexPoints();
  InitIntervalPoints();
  std::cout << " done!" << std::endl;
}  

void TIntPointEvaluator::InitIntervalPoints()
{
  // Gauss/Legendre, degree 1
  {
    Interval1_1.Insert(new IntPoint1D(2, 0));
  }

  // Gauss/Legendre, degree 3
  {
    Interval3_1.Insert(new IntPoint1D(1, -0.5773502692));
    Interval3_1.Insert(new IntPoint1D(1,  0.5773502692));
  }

  // Gauss/Legendre, degree 5
  {
    Interval5_1.Insert(new IntPoint1D(5.0/9, -0.7745966692));
    Interval5_1.Insert(new IntPoint1D(8.0/9,  0           ));
    Interval5_1.Insert(new IntPoint1D(5.0/9,  0.7745966692));
  }

  // Gauss/Legendre, degree 7
  {
    Interval7_1.Insert(new IntPoint1D(0.3478548451, -0.8611363116));
    Interval7_1.Insert(new IntPoint1D(0.6521451549, -0.3399810436));
    Interval7_1.Insert(new IntPoint1D(0.6521451549,  0.3399810436));
    Interval7_1.Insert(new IntPoint1D(0.3478548451,  0.8611363116));
  }

  // Gauss/Legendre, degree 9
  {
    Interval9_1.Insert(new IntPoint1D(0.2369268851, -0.9061798459));
    Interval9_1.Insert(new IntPoint1D(0.4786286705, -0.5384693101));
    Interval9_1.Insert(new IntPoint1D(128.0/225   ,  0           ));
    Interval9_1.Insert(new IntPoint1D(0.4786286705,  0.5384693101));
    Interval9_1.Insert(new IntPoint1D(0.2369268851,  0.9061798459));
  }
}
  
void TIntPointEvaluator::InitCubePoints()
{
  double V = 4;   // volume of 2-dimensional cube C_2 
  //int n=2;        // dimension
  
  // formula C_2: 3-1 (Stroud[71] p.243)
  {
  	double r = 1.0/3;
    double weight = V/4;
    Cube3_1.Insert(new IntPoint2D(weight, double2( r, r)));
    Cube3_1.Insert(new IntPoint2D(weight, double2(-r, r)));
    Cube3_1.Insert(new IntPoint2D(weight, double2( r,-r)));
    Cube3_1.Insert(new IntPoint2D(weight, double2(-r,-r)));
  }

  // formula C_2: 5-1 (Stroud[71] p.246)
  {
  	double r = sqrt(3.0/5);
  	double s = sqrt(1.0/3);
  	double t = sqrt(14.0/15);

	double A = V*(5.0/36);
	double B = V*(5.0/63);
	double C = V*(2.0/7);
  	
    Cube5_1.Insert(new IntPoint2D(A, double2( r, s)));
    Cube5_1.Insert(new IntPoint2D(A, double2(-r, s)));
    Cube5_1.Insert(new IntPoint2D(A, double2( r,-s)));
    Cube5_1.Insert(new IntPoint2D(A, double2(-r,-s)));
    Cube5_1.Insert(new IntPoint2D(B, double2( 0, t)));
    Cube5_1.Insert(new IntPoint2D(B, double2( 0,-t)));
    Cube5_1.Insert(new IntPoint2D(C, double2( 0, 0)));
  }

  // formula C_2: 7-1 (Stroud[71] p.252)
  {
  	double r = sqrt(6.0/7);
  	double s = sqrt( (114.0 - 3*sqrt(583)) / 287 );
  	double t = sqrt( (114.0 + 3*sqrt(583)) / 287 );

	double B1 = V*(49.0/810);
	double B2 = V*(178981 + 2769*sqrt(583))/1888920;
	double B3 = V*(178981 - 2769*sqrt(583))/1888920;
  	
    Cube7_1.Insert(new IntPoint2D(B1, double2( r, 0)));
    Cube7_1.Insert(new IntPoint2D(B1, double2(-r, 0)));
    Cube7_1.Insert(new IntPoint2D(B1, double2( 0, r)));
    Cube7_1.Insert(new IntPoint2D(B1, double2( 0,-r)));
    Cube7_1.Insert(new IntPoint2D(B2, double2( s, s)));
    Cube7_1.Insert(new IntPoint2D(B2, double2(-s, s)));
    Cube7_1.Insert(new IntPoint2D(B2, double2( s,-s)));
    Cube7_1.Insert(new IntPoint2D(B2, double2(-s,-s)));
    Cube7_1.Insert(new IntPoint2D(B3, double2( t, t)));
    Cube7_1.Insert(new IntPoint2D(B3, double2(-t, t)));
    Cube7_1.Insert(new IntPoint2D(B3, double2( t,-t)));
    Cube7_1.Insert(new IntPoint2D(B3, double2(-t,-t)));
  }

  // formula C_2: 9-1 (Stroud[71] p.258)
  {
    double r[4], s[4], B[4];
    
    r[0] = 0.9854398119422523;
    r[1] = 0.4888863428423724;
    r[2] = 0.9395672874215217;
    r[3] = 0.8367103250239890;
    s[0] = 0.0;
	s[1] = 0.0;
	s[2] = r[2];
	s[3] = 0.5073767736746132;
	B[0] = 0.0716134247098111;
	B[1] = 0.4540903525515453;
	B[2] = 0.0427846154667780;
	B[3] = 0.2157558036359328;

    Cube9_1.Insert(new IntPoint2D(B[0], double2( r[0], s[0])));     
    Cube9_1.Insert(new IntPoint2D(B[0], double2(-r[0], s[0])));     
    Cube9_1.Insert(new IntPoint2D(B[0], double2( s[0], r[0])));     
    Cube9_1.Insert(new IntPoint2D(B[0], double2( s[0],-r[0])));     

    Cube9_1.Insert(new IntPoint2D(B[1], double2( r[1], s[1])));     
    Cube9_1.Insert(new IntPoint2D(B[1], double2(-r[1], s[1])));     
    Cube9_1.Insert(new IntPoint2D(B[1], double2( s[1], r[1])));     
    Cube9_1.Insert(new IntPoint2D(B[1], double2( s[1],-r[1])));     

    Cube9_1.Insert(new IntPoint2D(B[2], double2( r[2], s[2])));     
    Cube9_1.Insert(new IntPoint2D(B[2], double2(-r[2], s[2])));     
    Cube9_1.Insert(new IntPoint2D(B[2], double2( r[2],-s[2])));     
    Cube9_1.Insert(new IntPoint2D(B[2], double2(-r[2],-s[2])));     

    Cube9_1.Insert(new IntPoint2D(B[3], double2( r[3], s[3])));     
    Cube9_1.Insert(new IntPoint2D(B[3], double2(-r[3], s[3])));     
    Cube9_1.Insert(new IntPoint2D(B[3], double2( r[3],-s[3])));     
    Cube9_1.Insert(new IntPoint2D(B[3], double2(-r[3],-s[3])));     
    Cube9_1.Insert(new IntPoint2D(B[3], double2( s[3], r[3])));     
    Cube9_1.Insert(new IntPoint2D(B[3], double2(-s[3], r[3])));     
    Cube9_1.Insert(new IntPoint2D(B[3], double2( s[3],-r[3])));     
    Cube9_1.Insert(new IntPoint2D(B[3], double2(-s[3],-r[3])));     
  }
}

void TIntPointEvaluator::InitSimplexPoints()
{
  double V = 1.0/2; // volume of 2-dimensional simplex T_2 
  int n=2;        // dimension
  
  // formula T_2: 2-1 (Stroud[71] p.307)
  {
    double r = ((n+2) - sqrt(n+2))/((n+1)*(n+2));
    double s = ((n+2) + n*sqrt(n+2))/((n+1)*(n+2));
    double weight = V/(n+1);
    Simplex2_2.Insert(new IntPoint2D(weight, double2(r,r)));
    Simplex2_2.Insert(new IntPoint2D(weight, double2(r,s)));
    Simplex2_2.Insert(new IntPoint2D(weight, double2(s,r)));
  }

  // formula T_2: 3-1 (Stroud[71] p.314)
  {
    double r = 1.0/2;
    double s = 0;
    double u = 1.0/6;
    double v = 4.0/6;

    double B = V/30;
    double C = 9*V/30;
    
    Simplex2_3.Insert(new IntPoint2D(B, double2(r,r)));
    Simplex2_3.Insert(new IntPoint2D(B, double2(r,s)));
    Simplex2_3.Insert(new IntPoint2D(B, double2(s,r)));
    Simplex2_3.Insert(new IntPoint2D(C, double2(u,u)));
    Simplex2_3.Insert(new IntPoint2D(C, double2(u,v)));
    Simplex2_3.Insert(new IntPoint2D(C, double2(v,u)));
  }

  // formula T_2: 5-1 (Stroud[71] p.314)
  {
    double r = (6 - sqrt(15))/21;
    double s = (9 + 2*sqrt(15))/21;
    double t = 1.0/3;
    double u = (6 + sqrt(15))/21;
    double v = (9 - 2*sqrt(15))/21;   

    double A = V*9.0/40;
    double B = V*(155 - sqrt(15))/1200;
    double C = V*(155 + sqrt(15))/1200;
    
    Simplex2_5.Insert(new IntPoint2D(A, double2(t,t)));
    Simplex2_5.Insert(new IntPoint2D(B, double2(r,r)));
    Simplex2_5.Insert(new IntPoint2D(B, double2(r,s)));
    Simplex2_5.Insert(new IntPoint2D(B, double2(s,r)));
    Simplex2_5.Insert(new IntPoint2D(C, double2(u,u)));
    Simplex2_5.Insert(new IntPoint2D(C, double2(u,v)));
    Simplex2_5.Insert(new IntPoint2D(C, double2(v,u)));
  }
   
  // formula T_2: 7-1 (Stroud[71] p.314)
  {
    double r[4], s[4], A[4], B[4];
    
    r[0] = 0.0694318422;   A[0] = 0.1739274226;
    r[1] = 0.3300094782;   A[1] = 0.3260725774;
    r[2] = 0.6699905218;   A[2] = 0.3260725774;
    r[3] = 0.9305681558;   A[3] = 0.1739274226;

    s[0] = 0.0571041961;   B[0] = 0.1355069134;
    s[1] = 0.2768430136;   B[1] = 0.2034645680;
    s[2] = 0.5835904324;   B[2] = 0.1298475476;
    s[3] = 0.8602401357;   B[3] = 0.0311809709;

    for (int i=0; i<4; i++)
      for (int j=0; j<4; j++)
        Simplex2_7.Insert(new IntPoint2D(A[i]*B[j], double2(s[j],r[i]*(1-s[j]))));
  }  
}

IntPoint1DList TIntPointEvaluator::GetIntervalIntPoints(const double2& interval, int degree)
{
  IntPoint1DList* intpoints = 0;
 
  switch (degree) {
    case 1:
      intpoints = &Interval1_1;
      break;
    case 3:
      intpoints = &Interval3_1;
      break;
    case 5:
      intpoints = &Interval5_1;
      break;
    case 7:
      intpoints = &Interval7_1;
      break;
    case 9:
      intpoints = &Interval9_1;
      break;
    default: {
      std::cerr << "Wrong degree in GetIntervalIntPoints()" << std::endl;
    }  
  }

  IntPoint1DList Result((*intpoints).Count, 1);

  double width = interval.y() - interval.x();
  double a = width/2;
  double b = a + interval.x();

  for (int i=0; i<(*intpoints).Count; i++)
    Result.Insert(new IntPoint1D(
      a * (*intpoints)[i].Weight(),
      a * (*intpoints)[i].Point() + b
    ));
  
  return Result;
}

IntPoint2DList TIntPointEvaluator::GetTriangleIntPoints(const Triangle2D& triangle, int degree)
{
  double2x2 A;
  double2 b;
  triangle.GetSimplexToTriangleMap(A,b);
  double J = fabs(A.Det());

  IntPoint2DList* intpoints = 0;
 
  switch (degree) {
    case 2:
      intpoints = &Simplex2_2;
      break;
    case 3:
      intpoints = &Simplex2_3;
      break;
    case 5:
      intpoints = &Simplex2_5;
      break;
    case 7:
      intpoints = &Simplex2_7;
      break;
    default: {
      std::cerr << "Wrong degree in GetTriangleIntPoints()" << std::endl;
    }  
  }

  IntPoint2DList Result((*intpoints).Count, 1);
 
  for (int i=0; i<(*intpoints).Count; i++)
    Result.Insert(new IntPoint2D(
      J * (*intpoints)[i].Weight(),
      A * (*intpoints)[i].Point() + b
    ));
    
  return Result;
}

IntPoint2DList TIntPointEvaluator::GetSimpleTriangleIntPoints(const Triangle2D& triangle, int n)
//
//                                     ...
//        
//        
//                              ...             ...
//        
//        
//                      x               x               ...
//                  (n-1,0,1)        (n-2,1,1)   
//        
//             x                 x                 x             ...
//          (n,0,0)          (n-1,1,0)         (n-2,2,0)
//
//--------------------------------------------------------------------------
//
//                      x
//               (n-i-1,i-j,j+1)
//
//             x               x
//        (n-i,i-j,j)     (n-i-1,i-j+1,j)
//
//                      x
//               (n-i,i-j+1,j-1)
//
{
  int NumPoints = n*n;                     // 1 + 3 + 5 + ...
  double area = triangle.Area();
  double weight = area/NumPoints;
  
  IntPoint2DList Result(NumPoints, 1, 1);

  double2 p0 = triangle[0];
  double2 p1 = triangle[1];
  double2 p2 = triangle[2]; 
 
  for (int i=0; i<n; i++) {
    int j;

    // two loops for the triangles
    // (n-i,i-j,j) (n-i-1,i-j+1,j) (n-i-1,i-j,j+1) and
    // (n-i,i-j,j) (n-i,i-j+1,j-1) (n-i-1,i-j+1,j)  

    for (j=0; j<=i; j++) {
      double2 u = ((3*n-3*i-2)*p0 + (3*i-3*j+1)*p1 + (3*j+1)*p2)/(3*n);
      Result.Insert(new IntPoint2D(weight,u));
    }
    for (j=1; j<=i; j++) {
      double2 u = ((3*n-3*i-1)*p0 + (3*i-3*j+2)*p1 + (3*j-1)*p2)/(3*n);
      Result.Insert(new IntPoint2D(weight,u));
    }
  }
    
  return Result;
}

IntPoint2DList TIntPointEvaluator::GetRectangleIntPoints(const Rectangle2D& rectangle, int degree)
{
  double2x2 A;
  double2 b;
  rectangle.GetCubeToRectangleMap(A,b);
  double J = fabs(A.Det());

  IntPoint2DList* intpoints = 0;
 
  switch (degree) {
    case 3:
      intpoints = &Cube3_1;
      break;
    case 5:
      intpoints = &Cube5_1;
      break;
    case 7:
      intpoints = &Cube7_1;
      break;
    case 9:
      intpoints = &Cube9_1;
      break;
    default: {
      std::cerr << "Wrong degree in GetRectangleIntPoints()" << std::endl;
    }  
  }

  IntPoint2DList Result((*intpoints).Count, 1, 1);
 
  for (int i=0; i<(*intpoints).Count; i++)
    Result.Insert(new IntPoint2D(
      J * (*intpoints)[i].Weight(),
      A * (*intpoints)[i].Point() + b
    ));
    
  return Result;
}

IntPoint2DList TIntPointEvaluator::GetPolygonIntPoints(const Polygon2D& polygon, int degree)
{
  List<Triangle2D> triangles;
  polygon.GetTriangulation(triangles);
  
  IntPoint2DList Result;
  
  for (int i=0; i<triangles.Count; i++)
    Result += GetTriangleIntPoints(triangles[i], degree);

  return Result;
}

